import logging
from logging.handlers import TimedRotatingFileHandler
import os
import socket
from datetime import datetime

LOG_FILE_BACKUP_COUNT = os.getenv('LOG_FILE_BACKUP_COUNT')
LOG_ROTATION = "midnight"

container_id = socket.gethostname()
timestamp = datetime.today().strftime('%Y-%m-%d')


def configure_logger():  # pragma: no cover
    log_level = os.getenv("APP_LOG_LEVEL", "WARNING")
    log_file_path = os.getenv("LOGFILE_PATH",
                              "/home/appuser/logs/rabbitmq-email-notifier")
    formatter = logging.Formatter(
                '%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    console_handler = logging.StreamHandler()
    console_handler.setFormatter(formatter)

    logger = logging.getLogger('rabbitmq-email-notifier')
    logger.addHandler(console_handler)
    # Defaults to console logging
    if os.getenv("CONSOLE_LOGGING_ONLY", "true") == "false":
        file_handler = TimedRotatingFileHandler(
            filename=f"{log_file_path}/{container_id}_console_{timestamp}.log",
            when=LOG_ROTATION,
            backupCount=LOG_FILE_BACKUP_COUNT
        )
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)

    logger.setLevel(log_level)



import os
import logging
from opentelemetry import trace
from opentelemetry.sdk.resources import Resource
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import BatchSpanProcessor
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import (
        OTLPSpanExporter)
from opentelemetry.sdk.resources import SERVICE_NAME

# tracing setup
JAEGER_NAME = os.getenv('JAEGER_NAME')
JAEGER_SERVICE_NAME = os.getenv('JAEGER_SERVICE_NAME')

resource = Resource(attributes={SERVICE_NAME: JAEGER_SERVICE_NAME})
provider = TracerProvider(resource=resource)
otlp_exporter = OTLPSpanExporter(endpoint=JAEGER_NAME, insecure=True)
span_processor = BatchSpanProcessor(otlp_exporter)
provider.add_span_processor(span_processor)
trace.set_tracer_provider(provider)
tracer = trace.get_tracer(__name__)

logger = logging.getLogger('rabbitmq-email-notifier')

"""
This is a template of a basic worker class for etds.

Since: 2023-05-23
Author: cgoines
"""


class Worker():
    version = None

    def __init__(self):
        self.version = os.getenv("APP_VERSION", "0.0.1")

    def get_version(self):
        return self.version

    # this is a stub call
    def call_api(self):
        with tracer.start_as_current_span("notifier_endpoint",
                                          attributes={"endpoint":
                                                      "/notifier"}):
            span = trace.get_current_span()
            span.add_event("log", {"notifier worker"})
            logger.debug("notifier worker")
            return True



import os
import sys
import time
from pathlib import Path
# heartbeat script for docker/k8s

hbeat_path = os.getenv("HEARTBEAT_FILE", "/tmp/worker_heartbeat")
HEARTBEAT_FILE = Path(hbeat_path)
HEARTBEAT_WINDOW = int(os.getenv("HEARTBEAT_WINDOW", 60))


# check timestamp file
try:
    current_ts = int(time.time())
    fstats = os.stat(HEARTBEAT_FILE)
    mtime = int(fstats.st_mtime)
    seconds_diff = int(current_ts - mtime)

    if (seconds_diff < HEARTBEAT_WINDOW):
        print("healthy: last updated %d secs ago" % (seconds_diff))
        sys.exit(0)
    else:
        print("UNHEALTHY: last updated %d secs ago" % (seconds_diff))
        sys.exit(1)

except Exception:
    print("Error: file not found")
    sys.exit(1)



from celery import Celery
from celery import bootsteps
from celery.signals import worker_ready
from celery.signals import worker_shutdown
from pathlib import Path
import os
import logging
import rabbitmqresources
import json
import traceback
from opentelemetry import trace
from opentelemetry.sdk.resources import Resource
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import BatchSpanProcessor
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import (
        OTLPSpanExporter)
from opentelemetry.sdk.resources import SERVICE_NAME
from opentelemetry.trace.propagation.tracecontext \
    import TraceContextTextMapPropagator

app = Celery()
app.config_from_object('celeryconfig')
rabbitmqresources.configure_logger()
logger = logging.getLogger('rabbitmq-email-notifier')

# tracing setup
JAEGER_NAME = os.getenv('JAEGER_NAME')
JAEGER_SERVICE_NAME = os.getenv('JAEGER_SERVICE_NAME')

resource = Resource(attributes={SERVICE_NAME: JAEGER_SERVICE_NAME})
provider = TracerProvider(resource=resource)
otlp_exporter = OTLPSpanExporter(endpoint=JAEGER_NAME, insecure=True)
span_processor = BatchSpanProcessor(otlp_exporter)
provider.add_span_processor(span_processor)
trace.set_tracer_provider(provider)
tracer = trace.get_tracer(__name__)


# heartbeat setup
# code is from
# https://github.com/celery/celery/issues/4079#issuecomment-1270085680
hbeat_path = os.getenv("HEARTBEAT_FILE", "/tmp/worker_heartbeat")
ready_path = os.getenv("READINESS_FILE", "/tmp/worker_ready")
update_interval = float(os.getenv("HEALTHCHECK_UPDATE_INTERVAL", 15.0))
HEARTBEAT_FILE = Path(hbeat_path)
READINESS_FILE = Path(ready_path)
UPDATE_INTERVAL = update_interval  # touch file every 15 seconds


class LivenessProbe(bootsteps.StartStopStep):
    requires = {'celery.worker.components:Timer'}

    def __init__(self, worker, **kwargs):  # pragma: no cover
        self.requests = []
        self.tref = None

    def start(self, worker):  # pragma: no cover
        self.tref = worker.timer.call_repeatedly(
            UPDATE_INTERVAL, self.update_heartbeat_file,
            (worker,), priority=10,
        )

    def stop(self, worker):  # pragma: no cover
        HEARTBEAT_FILE.unlink(missing_ok=True)

    def update_heartbeat_file(self, worker):  # pragma: no cover
        HEARTBEAT_FILE.touch()


@worker_ready.connect
def worker_ready(**_):  # pragma: no cover
    READINESS_FILE.touch()


@worker_shutdown.connect
def worker_shutdown(**_):  # pragma: no cover
    READINESS_FILE.unlink(missing_ok=True)


# return the number of seconds this message should expire in the future
def _get_expiration():
    # Default to one hour from now
    expiration = int(os.getenv('MESSAGE_EXPIRATION_SECS', 3600))
    return expiration


app.steps["worker"].add(LivenessProbe)


@app.task(serializer='json',
          name='rabbitmq-email-notifier.tasks.notify_email_message')
def notify_email_message(json_message):
    ctx = None
    if "traceparent" in json_message:  # pragma: no cover, tracing is not being tested # noqa: E501
        carrier = {"traceparent": json_message["traceparent"]}
        ctx = TraceContextTextMapPropagator().extract(carrier)
    with tracer.start_as_current_span("notify_email_message",
                                      context=ctx) as current_span:
        logger.info("message")
        logger.info(json_message)
        current_span.add_event(json.dumps(json_message))

        NEXT_TASK = os.getenv('NEXT_TASK')
        NEXT_QUEUE = os.getenv('NEXT_QUEUE_NAME')
        expiration = _get_expiration()

        body = json_message["body"]
        subject = json_message["subject"]
        recipients = json_message["recipients"]

        if (("exception" in json_message) and
                (json_message["exception"] is not None)):
            exc = traceback.format_exc(json_message["exception"])
            body = body + "\n\n" + exc

        msg_json = {
            "subject": subject,
            "body": body,
            "recipients": recipients
        }

        logger.debug("msg json:")
        logger.debug(msg_json)
        message = json.dumps(msg_json)
        current_span.add_event(message)

        # If only unit testing, return the message and
        # do not trigger the next task.
        if "unit_test" in json_message:
            return msg_json
        # preserve trace id across components
        carrier = {}  # pragma: no cover, tracing is not being tested # noqa: E501
        TraceContextTextMapPropagator().inject(carrier)  # pragma: no cover, tracing is not being tested # noqa: E501
        traceparent = carrier["traceparent"]  # pragma: no cover, tracing is not being tested # noqa: E501
        msg_json["traceparent"] = traceparent  # pragma: no cover, tracing is not being tested # noqa: E501
        current_span.add_event("to next queue")  # pragma: no cover, unit tests end before this span # noqa: E501
        logger.debug("MESSAGE TO QUEUE " + NEXT_QUEUE)
        logger.debug(message)
        app.send_task(NEXT_TASK,
                      args=[msg_json], kwargs={}, expires=expiration,
                      queue=NEXT_QUEUE)  # pragma: no cover, unit tests should not progress the message # noqa: E501



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="lts-rabbitmqemailnotifier",
    version="0.0.2",
    author="Chip Goines",
    author_email="chip_goines@harvard.edu",
    description="A set of utilities for placing a message " +
                "onto a queue for an emailer listener to receive",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/harvard-lts/rabbitmq-email-notifier",
    packages=setuptools.find_packages(),
    install_requires=[
        'celery==5.2.7',
        'certifi==2023.7.22',
        'coverage==7.2.6',
        'iniconfig==2.0.0',
        'lxml==4.9.2',
        'opentelemetry-api==1.19.0',
        'opentelemetry-sdk==1.19.0',
        'opentelemetry-exporter-jaeger==1.19.0',
        'opentelemetry-exporter-otlp-proto-grpc==1.19.0',
        'opentelemetry-semantic-conventions==0.40b0',
        'packaging==23.1',
        'pika==1.3.2',
        'pluggy==1.0.0',
        'project-paths==1.1.1',
        'pytest==7.3.1',
        'pytest-mock==3.10.0',
        'python-dotenv==1.0.0',
        'requests==2.31.0',
        'requests-mock==1.10.0',
        'retrying==1.3.4',
        'toml==0.10.2',
        'tomli==2.0.1',
        'urllib3==2.0.2'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    include_package_data=True,
    package_data={
        # Include all *.json files in any package
        "": ["*.json"],
    }
)



from celery import Celery
import os
from datetime import datetime

now = datetime.now()
timestamp = now.strftime("%Y%m%d%H%M")
job_ticket_id = f"{timestamp}_rabbitmq_email_notifier"

app1 = Celery('tasks')
app1.config_from_object('celeryconfig')

arguments = {"subject": "test subject",
             "body": "test message",
             "recipients": "john_harvard@harvard.edu"}

res = app1.send_task('rabbitmq-email-notifier.tasks.notify_email_message',
                     args=[arguments], kwargs={},
                     queue=os.getenv("CONSUME_QUEUE_NAME"))
